/*

VIII PC Magazine Bulgaria programming contest
Problem 1: CATALOG

Author: Peter Ivanov Petrov, 21, Sofia
        software developer / student
        e-mail: piettropro@bigfoot.com


  
---------------------

   AVL-,  ,
..        .  
   -      
  ,   -  . 
         O(log N),
 N      .

-    C ,    C++   
- .    .  
  - ,    inline, 
  STL.     STL    
 AVL-,       , 
   .      
,       , 
    (   -  
   STL).

     Microsoft Visual C++ 6.0, Borland
C++ 5.5, MinGW GCC 2.95.2-1  Windows  GCC 2.95.2  Linux. 
EXE-,      Visual C++,    
 C++ ,    ,    
SGI (http://www.stlport.org),    -.

P.S.    ,  .  , 
     .    
  new  delete,   ,  
.       
  ADD, DS, DN  DL,    - 
, -      -
 swapping  .

*/

//------- Modified & enhanced by V.Molotkov ---------------//
	

#define COMPACT_NODES //VM
	//Saves 4 bytes/node by combyning cl and bf in one unsigned int;
	//besides, it is about 10% quicker than original P.Petrov's code;
	//this happens, may be, because the node size now is (multiple of) 16?!
//#define HASH_TABLE //VM: placed in driver file, if at all
	//Hash table used to speed up repeating insert() and erase() operations;
#define PP_OPTIMIZE //PP's versions of search_max, search_min.

#include <cstdio>
#include <cstdlib>
#include <functional>
#include <memory>
#include <string>

using namespace std;
//namespace std{

#ifdef TEST
#ifdef HASH_TABLE
	int hits,collisions;//Hash table hits and collisions
#endif
#endif

///////////////////////////////////
// Simple memory allocator class:

template <class T> 
class pp_allocator {
public:
	static T*allocate(){T*p=alloc.allocate(1,hint);hint=p;return p; }
	static void deallocate(T*p){if(hint==p)hint=NULL;alloc.deallocate(p,1);}
private:
	static std::allocator<T> alloc;
	static T*hint;
};
template <class T> std::allocator<T> pp_allocator<T>::alloc;
template <class T> T *pp_allocator<T>::hint=NULL;

///////////////////////////////////////////////////////////
// Static stack class (faster than the dynamic STL stack)

template <class E,const int MAX_SIZE=128>
class static_stack {
public:
	static_stack() { count=0; }
	bool empty()const{return count==0;}
	int size()const{return count;}
	E&top(){return store[count-1];}
	const E&top()const{return store[count-1];}
	void push(const E& x) {store[count++]=x;}
	//void pop(){--count;}
	E&pop(){return store[--count];}//This is a bit more useful (VM)
private:
	E store[MAX_SIZE];
	int count;
};

///////////////////////////
// Indexed AVL Tree class

enum BfVal{ZERO,PLUS,MINUS};//Balance factor values;

template <class E>
class index_tree_node {
public:
	typedef index_tree_node<E> self_type;

	E data;
	self_type *l,*r;
#ifndef COMPACT_NODES	
	int cl;//Number of elements in left subtree + 1
	BfVal bf;//Balance factor
#else
	unsigned int cl:30, bf:2;//unsigned! otherwise very buggy!
#endif

	index_tree_node(const E&e=E()):data(e){l=r=NULL;cl=1;bf=ZERO;}

	void *operator new(size_t nBytes){return pp_allocator<self_type>::allocate();}
	void operator delete(void*p){pp_allocator<self_type>::deallocate((self_type*)p);}
};

template <class E,class Pred=std::less<E> >
class index_tree{
public:
	typedef index_tree<E,Pred> self_type;
	typedef index_tree_node<E> node_type;

	index_tree():root(NULL),count(0)
#ifdef HASH_TABLE
		,hash(NULL),hstatus(NULL),hsize(0),INVALID(E())
#endif
	{}
	~index_tree(){erase(root);}

	int size()const{return count;}

	bool insert(const E&e){
#ifdef HASH_TABLE
		if(hinsert(e)==IN__)return false;
#endif
		node_type *p=root,*pp=NULL,*a=NULL,*pa;
		while(p){
			if(p->bf)a=p,pa=pp; 
			pp=p;
			if(Pred()(e,p->data))++p->cl,p=p->l;	
			else 
			if(Pred()(p->data,e))p=p->r;
			else{reset_c(-1,e);return false;}
		}
		count++;
		node_type *r=new node_type(e);
		if(!root){root=r;return true;}
		if(Pred()(e,pp->data))pp->l=r;else pp->r=r; 
		if(a){
			if(a->bf==MINUS)
				if (Pred()(e,a->data))a->bf=ZERO,fix_bf(a->l,r,e);
				else{
					node_type *b=a->r;
					if(Pred()(b->data,e))fix_bf(b->r,r,e),rotate_rr(pa,a,b);
					else fix_bf(b->l,r,e),rotate_rl(pa,a,b);
				}
			else 
				if(Pred()(a->data,e))a->bf=ZERO,fix_bf(a->r,r,e);
				else{
					node_type *b=a->l;
					if(Pred()(e,b->data))fix_bf(b->l,r,e),rotate_ll(pa,a,b);
					else fix_bf(b->r,r,e),rotate_lr(pa,a,b);
				}
		}
		else fix_bf(root,r,e);
		return true;
	}

	bool erase(int k,E*e=NULL){
		if(k<0||k>=count)return false;
		++k;
		static_stack<node_type*,128> st;
		node_type *p=root;
		while(p&&p->cl!=k){
			st.push(p); 
			if (k<p->cl)--p->cl,p=p->l; 
			else k-=p->cl,p=p->r; 
		}
#if 0	//switched off by VM	
		if(!p){//Is it possible?! It seems that not. And if yes, the code is inadequate! (VM)
			p=root; 
			while(p) 
				if(k<p->cl)--p->cl,p=p->l;
				else k-=p->cl,p=p->r; 
				return false; 
		}
#endif
		--count; 
		if(e)*e=p->data;
		if(p->l&&p->r){
			node_type *s=p->l;
			st.push(p);
			while(s->r)st.push(s),s=s->r;
			p->data=s->data; --p->cl; p=s;
		}
		node_type *c;
		if(p->l)c=p->l;else c=p->r;
		if(p==root)root=c;
		else 
		if(p==st.top()->l)st.top()->l=c; 
		else st.top()->r=c;
		E f=p->data; 
		delete p; 
		if(st.empty())return true;
		node_type *q=st.pop();
		while(q) 
			if(!Pred()(q->data,f)){
				//--q->bf; 
				if (q->bf==ZERO){q->bf=MINUS;return true;}
				if (q->bf==MINUS) {
					node_type *b=q->r,*pa;
					if(st.empty()) pa=NULL;else {pa=st.pop();}
					switch(b->bf){
						case 0: 
							rotate_rr(pa,q,b); 
							b->bf=PLUS; q->bf=MINUS; 
							return true;
						case 1: 
							rotate_rl(pa,q,b); break;
						default: 
							rotate_rr(pa,q,b);
					}
					q=pa;
				} 
				else{
					q->bf=ZERO;
					if(st.empty())return true; 
					q=st.pop(); 
				}
			} 
			else{
				//++q->bf; 
				if(q->bf==ZERO){q->bf=PLUS;return true;}
				if(q->bf==PLUS){
					node_type *b=q->l,*pa;
					if(st.empty())pa=NULL;else pa=st.pop();
					switch(b->bf){
						case 0: 
							rotate_ll(pa,q,b); 
							b->bf=MINUS; q->bf=PLUS; 
							return true;
						case 1: 
							rotate_ll(pa,q,b); 
							break;
						default: 
							rotate_lr(pa,q,b);
					}
					q=pa;
				} 
				else{
					q->bf=ZERO;
					if(st.empty())return true; 
					q=st.pop(); 
				}
			}
		return true;
	}

	//added by VM:
	bool erase(const E&e){
#ifdef HASH_TABLE
		if(herase(e)==OUT__)return false;
#endif
		static_stack<node_type*,128> st;
		node_type *p=root;
		while(p&&p->data!=e){
			st.push(p); 
			if (Pred()(e,p->data))--p->cl,p=p->l; 
			else p=p->r; 
		}
		if(!p){
		//Nothing to delete; restore original cl's:
			reset_c(1,e);
			return false; 
		}
		--count; 
		if(p->l&&p->r){
			node_type *s=p->l;
			st.push(p);
			while(s->r)st.push(s),s=s->r;
			p->data=s->data; --p->cl; p=s;
		}
		node_type*c;
		if(p->l)c=p->l;else c=p->r;
		if(p==root)root=c;
		else 
		if(p==st.top()->l)st.top()->l=c; 
		else st.top()->r=c;
		E f=p->data; 
		delete p; 
		if(st.empty())return true;
		node_type *q=st.pop();
		while(q) 
			if(!Pred()(q->data,f)){
				if (q->bf==ZERO){q->bf=MINUS;return true;}
				if (q->bf==MINUS) {
					node_type *b=q->r,*pa;
					if(st.empty())pa=NULL;else pa=st.pop();
					switch(b->bf){
						case 0: 
							rotate_rr(pa,q,b); 
							b->bf=PLUS; q->bf=MINUS; 
							return true;
						case 1: 
							rotate_rl(pa,q,b); break;
						default: 
							rotate_rr(pa,q,b);
					}
					q=pa;
				} 
				else{
					q->bf=ZERO;
					if(st.empty())return true;
					q=st.pop(); 
				}
			} 
			else{
				if(q->bf==ZERO){q->bf=PLUS;return true;}
				if(q->bf==PLUS){
					node_type *b=q->l,*pa;
					if(st.empty())pa=NULL;else pa=st.pop();
					switch(b->bf){
						case 0: 
							rotate_ll(pa,q,b); 
							b->bf=MINUS; q->bf=PLUS; 
							return true;
						case 1: 
							rotate_ll(pa,q,b); 
							break;
						default: 
							rotate_lr(pa,q,b);
					}
					q=pa;
				} 
				else{
					q->bf=ZERO;
					if(st.empty())return true;
					q=st.pop(); 
				}
			}
		return true;
	}

	int search(const E&e,E*exact=NULL)const{//index, or -1 if not found
		int k=0; node_type *p=root;
		while(p)
			if(Pred()(e,p->data))p=p->l; 
			else 
			if(Pred()(p->data,e))k+=p->cl,p=p->r;
			else{if(exact)*exact=p->data;return k+p->cl-1;}
		return -1;
	}

	//Added by VM:
	//returns index(&value) of minimal e'>=e
	int search_min(const E&e,E*exact)const{
		int k=0,i=-1; node_type*p=root,*q=NULL;
		while(p)
			if(Pred()(e,p->data))i=k+p->cl-1,q=p,p=p->l;//??
			else 
			if(Pred()(p->data,e))k+=p->cl,p=p->r;
			else{i=k+p->cl-1;q=p;break;}
		if(exact&&q)*exact=q->data;
		return i;
	}
	//returns index(&value) of maximal e'<=e
	int search_max(const E&e,E*exact)const{
		int k=0,i=-1; node_type*p=root,*q=NULL;
		while(p)
			if(Pred()(e,p->data))p=p->l; 
			else 
			if(Pred()(p->data,e))k+=p->cl,i=k,q=p,p=p->r;
			else{i=k+p->cl-1;q=p;break;}
		if(exact&&q)*exact=q->data;
		return i;
	}
	//A bit quicker specializations of above:
	//returns index of minimal e'>=e
#ifndef PP_OPTIMIZE
	int search_min(const E&e)const{
		int k=0,i=-1; node_type*p=root;
		while(p){
			if(Pred()(e,p->data))i=k+p->cl-1,p=p->l;//??
			else 
			if(Pred()(p->data,e))k+=p->cl,p=p->r;
			else{i=k+p->cl-1;break;}
		}
		return i;
	}
	//returns index of maximal e'<=e
	int search_max(const E&e)const{
		int k=0,i=-1; node_type*p=root;
		while(p){
			if(Pred()(e,p->data))p=p->l; 
			else 
			if(Pred()(p->data,e))k+=p->cl,i=k/*+p->cl*/-1,p=p->r;//??
			else{i=k+p->cl-1;break;}
		}
		return i;
	}
#else //This "non-symmetric" version of P.Petrov must be quicker a bit:
			//because it uses less assignements
	int search_min(const E&e)const{
		return search_max(e-1)+1;
	}
	//returns index of maximal e'<=e
	int search_max(const E&e)const{
		int k=-1; node_type*p=root;
		while(p){
			if(Pred()(e,p->data))p=p->l; 
			else 
			if(Pred()(p->data,e))k+=p->cl,p=p->r;
			else{return k+p->cl;}
		}
		return k;
	}
#endif

	bool at(int k,E*e)const{
		if(k<0||k>=count)return false; 
		++k;
		node_type *p=root;
		while(p) 
			if(k<p->cl)p=p->l; 
			else 
			if(k>p->cl)k-=p->cl,p=p->r;
			else{if(e)*e=p->data;return true;}
		return false;
	}

protected:
	node_type *root;
	int count;
#ifdef HASH_TABLE
public:
	E* hash,INVALID;unsigned*hstatus;
	int hsize;
protected:
	//In fact, the next function must be included as template,
	//but I have no time for this now:
	//unsigned int hval(const E&e){return (unsigned int)(e%hsize);}//for prime hsize
	unsigned int hval(const E&e){return (unsigned int)(e&(hsize-1));}//for hsize==power of 2
	enum status{IN__,OUT__,UNKNOWN__};
	status hinsert(const E&e){
		if(hash==NULL)return UNKNOWN__;
		register unsigned int hi=hval(e),si=hi>>5,mask=(1<<(hi&31));
		if(hash[hi]==e)
			if(hstatus[si]&mask){
#ifdef TEST
				hits++;
#endif
				return IN__;
			}
			else{
				hstatus[si]|=mask;
				return OUT__;
			}
		status st=(hash[hi]==INVALID)?OUT__:UNKNOWN__;
#ifdef TEST
		if(hash[hi]!=INVALID)collisions++;
#endif
		//Kick out previous item (if present), replace it by e:
		hash[hi]=e;hstatus[si]|=mask;//New status==IN
		return st;
	}
	status herase(const E&e){
		if(hash==NULL)
			return UNKNOWN__;
		register unsigned int hi=hval(e),si=hi>>5,mask=(1<<(hi&31));
		if(hash[hi]==e)
			if((hstatus[si]&mask)==0){
#ifdef TEST
				hits++;
#endif
				return OUT__;
			}
			else{
				hstatus[si]&=~mask;
				return IN__;
			}
		status st=(hash[hi]==INVALID)?OUT__:UNKNOWN__;
#ifdef TEST
		if(hash[hi]!=INVALID)collisions++;
		else hits++;
#endif
		//Kick out previous item (if present), replace it by e:
		hash[hi]=e,hstatus[si]&=~mask;//New status==OUT
		return st;
	}
#endif

	void erase(node_type*p){if(p){erase(p->l);erase(p->r);delete p;}}

	void fix_bf(node_type*q,node_type*r,const E&e){
		while(q!=r) 
			if(Pred()(e,q->data))q->bf=PLUS,q=q->l;
			else q->bf=MINUS,q=q->r; 
	}

	void rotate_ll(node_type*pa,node_type*a,node_type*b){
		a->l=b->r;b->r=a;a->cl-=b->cl;
		if(pa)
			if(a==pa->l)pa->l=b; 
			else pa->r=b; 
		else root=b;
		a->bf=b->bf=ZERO;
	}

	void rotate_rr(node_type*pa,node_type*a,node_type*b){
		a->r=b->l;b->l=a;b->cl+=a->cl;
		if(pa) 
			if(a==pa->l)pa->l=b; 
			else pa->r=b; 
		else root=b;
		a->bf=b->bf=ZERO;
	}

	void rotate_lr(node_type*pa,node_type*a,node_type*b){
		node_type*c=b->r;
		a->l=c->r;b->r=c->l;c->l=b;c->r=a;a->cl-=b->cl+c->cl;c->cl+=b->cl;
		if(pa) 
			if(a==pa->l)pa->l=c; 
			else pa->r=c; 
		else root=c;
		if(c->bf==ZERO)b->bf=a->bf=ZERO; 
		else{
			if(c->bf==PLUS)b->bf=ZERO,a->bf=MINUS; 
			else b->bf=PLUS,a->bf=ZERO;	
			c->bf=ZERO; 
		}
	}

	void rotate_rl(node_type*pa,node_type*a,node_type*b){
		node_type*c=b->l;
		a->r=c->l;b->l=c->r;c->l=a;c->r=b;b->cl-=c->cl;c->cl+=a->cl;
		if(pa) 
			if(a==pa->l)pa->l=c; 
			else pa->r=c; 
		else root=c;
		if(c->bf==ZERO)b->bf=a->bf=ZERO; 
		else{ 
			if(c->bf==PLUS)b->bf=MINUS,a->bf=ZERO;  
			else b->bf=ZERO,a->bf=PLUS;	
			c->bf=ZERO; 
		}
	}

	void reset_c(int r,const E&e) {
		node_type *p=root;
		while(p) 
			if(Pred()(e,p->data))p->cl+=r,p=p->l;  
			else 
			if(Pred()(p->data,e))p=p->r; 
			else return;
	}

};

//}//namespace std