 /***********************************************************
  *
  *   Program:    STAT (test 2 of PC Magazine(BG)
	*                       programmers contest 2002)
	*		Version:		2.0 (Uses memory mapped files for IO operations)
  *   Created:    05/01/03 19:54:PM 
  *   Author:     Vladimir Molotkov
	*   e-mail:     vmolot@inrne.bas.bg
	*
	*   Uses:       Peter Petrov's code (in avl.h) from Test 1 of 
	*               PC Magazine(BG) Programmers contest 2000;
	*								SGI's STL instead of MS STL -- it is REALLY much quicker
	*								(see comments below)
  *
  ************************************************************
  *
  ************************************************************/

//#define HASH_TABLE //VM
	//Hash table used to eventually speed up repeating insert() and erase();
	//But if the number of such operations is <=25%, there is slow down of about 10%
	//So it will not be used here

#pragma warning(disable:4786)

#include <stdio.h>
#include <stdlib.h>
#include <functional>
#include <memory>
#include <string>
#include <windows.h>
#include "avl.h"

using namespace std;

typedef unsigned long uint;

#ifdef HASH_TABLE
//const int H_SIZE=8388593;//4194301;//Prime: too slow (by 10%)
const int H_SIZE=2*1024*1024;//Power of 2: slow all the same
uint Hash[H_SIZE];
unsigned int Hstatus[H_SIZE/32+1];
#endif

int main(int argc,char**argv) {
	HANDLE fi,fo,fim,fom;
	const char*inptr,*inptr0;
	char*outptr0,*outptr;
  index_tree<unsigned> store;
	if((fi=CreateFile("STAT.INP",GENERIC_READ,0,NULL,OPEN_EXISTING,
		FILE_ATTRIBUTE_NORMAL|FILE_FLAG_RANDOM_ACCESS,NULL))
		==INVALID_HANDLE_VALUE){
		printf("Cannot open STAT.INP!\n");
		exit(1);
	}
	int filen=GetFileSize(fi,NULL);
	fim=CreateFileMapping(fi,NULL,PAGE_READONLY,0,0,NULL);
	CloseHandle(fi);
	inptr=inptr0=(const char*)MapViewOfFile(fim,FILE_MAP_READ,0,0,0);

	if((fo=CreateFile("STAT.OUT",GENERIC_WRITE|GENERIC_READ,0,NULL,CREATE_ALWAYS,
		FILE_ATTRIBUTE_NORMAL|FILE_FLAG_RANDOM_ACCESS,NULL))
		==INVALID_HANDLE_VALUE){
		printf("Cannot open STAT.OUT\n");
		exit(2);
	}
	if((fom=CreateFileMapping(fo,NULL,PAGE_READWRITE,0,filen/2,NULL))==NULL){
		printf("Cannot create file mapping!\n");
		exit(3);
	}
	outptr=outptr0=(char*)MapViewOfFile(fom,FILE_MAP_WRITE,0,0,filen/2);

//#define COMPARE_PP //Compares exact diff. in effectivity between PP's string buffer and memory mapped files
	//while(1) loop is taken from PP source almost verbatim,
	//but internal while's over spaces are excluded in !COMPARE_PP mode"
	while(1){
    char c,cmd;
    while((cmd=*inptr++)<'1'||cmd>'4');
    if(cmd=='1'){
#ifdef COMPARE_PP  
      while(*inptr==' ')//Superfluous: always executes only once!
#endif
				++inptr;
      unsigned n=0;
      while((c=*inptr++)>='0'&&c<='9')n=n*10+(c-'0');
      if (store.insert(n))
        *outptr++='1',*outptr++='\r',*outptr++='\n';
      else
        *outptr++='0',*outptr++='\r',*outptr++='\n';
    } 
		else if(cmd=='2'){
#ifdef COMPARE_PP  
      while(*inptr==' ')//Superfluous: always executes only once!
#endif
				++inptr;
      unsigned n = 0;
      while((c=*inptr++)>='0'&&c<='9')n=n*10+(c-'0');
      if (store.erase(n)) 
        *outptr++='1',*outptr++='\r',*outptr++='\n';
      else
        *outptr++='0',*outptr++='\r',*outptr++='\n';
    } 
		else if(cmd=='3'){
#ifdef COMPARE_PP  
      while(*inptr==' ')//Superfluous: always executes only once!
#endif
				++inptr;
      unsigned m=0,n=0;
      while((c=*inptr++)>='0'&&c<='9')m=m*10+(c-'0');
#ifdef COMPARE_PP  
			while(*inptr==' ')++inptr;//This loop is never executed! 
#endif
      while((c=*inptr++)>='0'&&c<='9')n=n*10+(c-'0');
      //int count = store.count_less(n+1) - store.count_less(m);
			int count=store.search_max(n)-store.search_min(m)+1;
      int base=1;
      while(base<=count)base*=10;
      if(!count)*outptr++='0';
      while(base>1){
        base /= 10;
        *outptr++=char(count/base)+'0';
        count%=base;
      }
      *outptr++='\r',*outptr++='\n';
    } 
		else break;
  }
  
#if 1
//Try to fake system time:	
	if(argc>1){
		int deltah=atoi(argv[1]);
		SYSTEMTIME st;
		GetSystemTime(&st);       // gets current time
		//st.wMilliseconds = 0;  // "round" milliseconds 
		//if(st.wSecond>0)st.wSecond -= 1;
		st.wHour += deltah;
		if (!SetSystemTime(&st))  // sets system time
			printf("Cannot change system time!\n");
	}
#endif

	UnmapViewOfFile(inptr0);
	UnmapViewOfFile(outptr0);
	CloseHandle(fim);
	CloseHandle(fom);
	SetFilePointer(fo,outptr-outptr0,NULL,FILE_BEGIN);
	SetEndOfFile(fo);
	CloseHandle(fo);

  return 0;
}
